﻿using Borepin.ViewModel;
using System.Collections.Generic;
using System.Threading.Tasks;
using System.Windows.Input;
using Prism.Commands;
using Prism.Navigation;
using Borepin.Service.BFFH;
using Borepin.Base;
using FabAccessAPI.Schema;
using Prism.Services.Dialogs;
using Prism.Services;
using Prism.AppModel;

namespace Borepin.PageModel
{
    public class MachineListPageModel : ConnectionModelBase, IPageLifecycleAware
    {
        #region Private Fields
        private readonly IDialogService _DialogService;

        private Machine _NextMachine;
        #endregion

        #region Constructors
        public MachineListPageModel(INavigationService navigationService, IPageDialogService pageDialogService, IBFFHService bFFHService, IDialogService dialogService) : base(navigationService, pageDialogService, bFFHService)
        {
            _DialogService = dialogService;

            SelectInstanceCommand = new DelegateCommand<object>(SelectInstanceCommandExecute);
            ScanCodeCommand = new DelegateCommand(ScanCodeCommandExecute);
            RefreshCommand = new DelegateCommand(async ()=> await RefreshCommandExecute().ConfigureAwait(true));
        }
        #endregion

        #region Data
        public override async Task LoadData()
        {
            if (! await CheckConnection().ConfigureAwait(false))
            {
                IsConnected = false;

                IsBusy = false;
                return;
            }

            IsConnected = true;

            IMachineSystem machineInterface = await _BFFHService.GetMachineSystemInterface().ConfigureAwait(false);

            MachineSystem.IInfoInterface infoInterface = await machineInterface.Info().ConfigureAwait(false);

            IReadOnlyList<Machine> machine_list = await infoInterface.GetMachineList().ConfigureAwait(false);

            List<MachineListItemViewModel> viewmodel_list = new List<MachineListItemViewModel>();
            foreach (Machine machine in machine_list)
            {
                viewmodel_list.Add(new MachineListItemViewModel(machine));
            }

            MachineListItemViewModel_List = viewmodel_list;

            IsBusy = false;
        }
        #endregion

        #region Fields
        private IList<MachineListItemViewModel> _MachineListItemViewModel_List;
        public IList<MachineListItemViewModel> MachineListItemViewModel_List
        {
            get => _MachineListItemViewModel_List;
            set => SetProperty(ref _MachineListItemViewModel_List, value);
        }

        private bool _IsRefreshing;
        public bool IsRefreshing
        {
            get => _IsRefreshing;
            set => SetProperty(ref _IsRefreshing, value);
        }
        #endregion

        #region Commands
        private ICommand _RefreshCommand;
        public ICommand RefreshCommand
        {
            get => _RefreshCommand;
            set => SetProperty(ref _RefreshCommand, value);
        }
        public async Task RefreshCommandExecute()
        {
            await LoadData().ConfigureAwait(true);
            IsRefreshing = false;
        }

        private ICommand _SelectInstanceCommand;
        public ICommand SelectInstanceCommand
        {
            get => _SelectInstanceCommand;
            set => SetProperty(ref _SelectInstanceCommand, value);
        }
        public async void SelectInstanceCommandExecute(object obj)
        {
            MachineListItemViewModel viewmodel = obj as MachineListItemViewModel;

            NavigationParameters parameters = new NavigationParameters
            {
                { "id", viewmodel.Instance.Id },
            };

            await _NavigationService.NavigateAsync($"MachinePage", parameters).ConfigureAwait(false);
        }

        private ICommand _ScanCodeCommand;
        public ICommand ScanCodeCommand
        {
            get => _ScanCodeCommand;
            set => SetProperty(ref _ScanCodeCommand, value);
        }
        public void ScanCodeCommandExecute()
        {
            IsBusy = true;

            _DialogService.ShowDialog("ScanDialog", ScanCodeCommandExecute_Dialog);
        }

        public async void ScanCodeCommandExecute_Dialog(IDialogResult result)
        {
            if (string.Equals(result.Parameters.GetValue<string>("result"), "scanned", System.StringComparison.Ordinal))
            {
                if (! _BFFHService.IsConnected)
                {
                    IsConnected = false;
                    IsBusy = false;
                    return;
                }

                IMachineSystem machineInterface = await _BFFHService.GetMachineSystemInterface().ConfigureAwait(false);
                MachineSystem.IInfoInterface infoInterface = await machineInterface.Info().ConfigureAwait(false);

                Machine machine = (await infoInterface.GetMachineURN(result.Parameters.GetValue<string>("value")).ConfigureAwait(false)).Item1;

                if(machine == null)
                {
                    IsBusy = false;
                    return;
                }

                _NextMachine = machine;
                IsBusy = false;
            }
        }
        #endregion

        #region IPageLifecycleAware
        public async void OnAppearing()
        {
            if(_NextMachine != null)
            {
                NavigationParameters parameters = new NavigationParameters
                {
                    { "id", _NextMachine.Id},
                };

                _NextMachine = null;
                
                await _NavigationService.NavigateAsync("MachinePage", parameters).ConfigureAwait(false);
            }
        }

        public void OnDisappearing()
        {

        }
        #endregion

        #region INavigationAware
        public override void OnNavigatedFrom(INavigationParameters parameters)
        {
            
        }

        public override async void OnNavigatedTo(INavigationParameters parameters)
        {
            await LoadData().ConfigureAwait(false);
        }
        #endregion
    }
}
